@@ -384,10 +384,9 @@ fn find_candidate(backtrack_stack: &mut Vec<BacktrackFrame>,
             return Some(candidate)
         }
     }
-    return None
+    None
 }
 
-#[allow(deprecated)] // connect => join in 1.3
 fn activation_error(cx: &Context,
                     registry: &mut Registry,
                     parent: &Summary,
@@ -424,7 +423,7 @@ fn activation_error(cx: &Context,
                                         .map(|v| v.version())
                                         .map(|v| v.to_string())
                                         .collect::<Vec<_>>()
-                                        .connect(", ")));
+                                        .join(", ")));
 
         return human(msg)
     }
@@ -453,7 +452,7 @@ fn activation_error(cx: &Context,
     candidates.sort_by(|a, b| {
         b.version().cmp(a.version())
     });
-    if candidates.len() > 0 {
+    if !candidates.is_empty() {
         msg.push_str("\nversions found: ");
         for (i, c) in candidates.iter().take(3).enumerate() {
             if i != 0 { msg.push_str(", "); }
@@ -469,7 +468,7 @@ fn activation_error(cx: &Context,
     // update`. In this case try to print a helpful error!
     if dep.source_id().is_path() &&
        dep.version_req().to_string().starts_with("=") &&
-       candidates.len() > 0 {
+       !candidates.is_empty() {
         msg.push_str("\nconsider running `cargo update` to update \
                       a path dependency's locked version");
 
@@ -607,7 +606,7 @@ impl Context {
                     (!use_default || prev.contains("default") ||
                      !has_default_feature)
             }
-            None => features.len() == 0 && (!use_default || !has_default_feature)
+            None => features.is_empty() && (!use_default || !has_default_feature)
         }
     }
 
@@ -648,7 +647,6 @@ impl Context {
         self.activations.get(&key).map(|v| &v[..]).unwrap_or(&[])
     }
 
-    #[allow(deprecated)] // connect => join in 1.3
     fn resolve_features(&mut self, parent: &Summary, method: &Method)
             -> CargoResult<Vec<(Dependency, Vec<String>)>> {
         let dev_deps = match *method {
@@ -686,18 +684,18 @@ impl Context {
         // they should have all been weeded out by the above iteration. Any
         // remaining features are bugs in that the package does not actually
         // have those features.
-        if feature_deps.len() > 0 {
+        if !feature_deps.is_empty() {
             let unknown = feature_deps.keys().map(|s| &s[..])
                                       .collect::<Vec<&str>>();
-            if unknown.len() > 0 {
-                let features = unknown.connect(", ");
+            if !unknown.is_empty() {
+                let features = unknown.join(", ");
                 bail!("Package `{}` does not have these features: `{}`",
                       parent.package_id(), features)
             }
         }
 
         // Record what list of features is active for this package.
-        if used_features.len() > 0 {
+        if !used_features.is_empty() {
             let pkgid = parent.package_id();
             self.resolve.features.entry(pkgid.clone())
                 .or_insert(HashSet::new())
