@@ -106,7 +106,7 @@ impl<'cfg> PathSource<'cfg> {
         let mut filter = |p: &Path| {
             let relative_path = util::without_prefix(p, &root).unwrap();
             include.iter().any(|p| p.matches_path(&relative_path)) || {
-                include.len() == 0 &&
+                include.is_empty() &&
                  !exclude.iter().any(|p| p.matches_path(&relative_path))
             }
         };
@@ -244,7 +244,7 @@ impl<'cfg> PathSource<'cfg> {
             let loc = pkg.root();
             try!(PathSource::walk(loc, &mut ret, true, filter));
         }
-        return Ok(ret);
+        Ok(ret)
     }
 
     fn walk(path: &Path, ret: &mut Vec<PathBuf>,
@@ -275,7 +275,7 @@ impl<'cfg> PathSource<'cfg> {
             }
             try!(PathSource::walk(&dir, ret, false, filter));
         }
-        return Ok(())
+        Ok(())
     }
 }
 
@@ -302,18 +302,13 @@ impl<'cfg> Source for PathSource<'cfg> {
         Ok(())
     }
 
-    fn download(&mut self, _: &[PackageId])  -> CargoResult<()>{
-        // TODO: assert! that the PackageId is contained by the source
-        Ok(())
-    }
+    fn download(&mut self, id: &PackageId) -> CargoResult<Package> {
+        trace!("getting packages; id={}", id);
 
-    fn get(&self, ids: &[PackageId]) -> CargoResult<Vec<Package>> {
-        trace!("getting packages; ids={:?}", ids);
-
-        Ok(self.packages.iter()
-           .filter(|pkg| ids.iter().any(|id| pkg.package_id() == id))
-           .map(|pkg| pkg.clone())
-           .collect())
+        let pkg = self.packages.iter().find(|pkg| pkg.package_id() == id);
+        pkg.cloned().ok_or_else(|| {
+            internal(format!("failed to find {} in path source", id))
+        })
     }
 
     fn fingerprint(&self, pkg: &Package) -> CargoResult<String> {
