@@ -2,18 +2,18 @@ use std::cell::{RefCell, RefMut, Ref, Cell};
 use std::collections::hash_map::Entry::{Occupied, Vacant};
 use std::collections::hash_map::{HashMap};
 use std::env;
-use std::ffi::OsString;
 use std::fmt;
 use std::fs::{self, File};
 use std::io::prelude::*;
 use std::mem;
 use std::path::{Path, PathBuf};
+use std::str::FromStr;
 
 use rustc_serialize::{Encodable,Encoder};
 use toml;
 use core::shell::{Verbosity, ColorConfig};
 use core::{MultiShell, Package};
-use util::{CargoResult, ChainError, Rustc, internal, human, paths};
+use util::{CargoResult, CargoError, ChainError, Rustc, internal, human, paths};
 
 use util::toml as cargo_toml;
 
@@ -118,7 +118,7 @@ impl Config {
         *self.target_dir.borrow_mut() = Some(path.to_owned());
     }
 
-    pub fn get(&self, key: &str) -> CargoResult<Option<ConfigValue>> {
+    fn get(&self, key: &str) -> CargoResult<Option<ConfigValue>> {
         let vals = try!(self.values());
         let mut parts = key.split('.').enumerate();
         let mut val = match vals.get(parts.next().unwrap().1) {
@@ -149,50 +149,115 @@ impl Config {
         Ok(Some(val.clone()))
     }
 
-    pub fn get_string(&self, key: &str) -> CargoResult<Option<(String, PathBuf)>> {
+    fn get_env<V: FromStr>(&self, key: &str) -> CargoResult<Option<Value<V>>>
+        where Box<CargoError>: From<V::Err>
+    {
+        let key = key.replace(".", "_")
+                     .replace("-", "_")
+                     .chars()
+                     .flat_map(|c| c.to_uppercase())
+                     .collect::<String>();
+        match env::var(&format!("CARGO_{}", key)) {
+            Ok(value) => {
+                Ok(Some(Value {
+                    val: try!(value.parse()),
+                    definition: Definition::Environment,
+                }))
+            }
+            Err(..) => Ok(None),
+        }
+    }
+
+    pub fn get_string(&self, key: &str) -> CargoResult<Option<Value<String>>> {
+        if let Some(v) = try!(self.get_env(key)) {
+            return Ok(Some(v))
+        }
         match try!(self.get(key)) {
-            Some(CV::String(i, path)) => Ok(Some((i, path))),
+            Some(CV::String(i, path)) => {
+                Ok(Some(Value {
+                    val: i,
+                    definition: Definition::Path(path),
+                }))
+            }
             Some(val) => self.expected("string", key, val),
             None => Ok(None),
         }
     }
 
-    pub fn get_path(&self, key: &str) -> CargoResult<Option<PathBuf>> {
-        if let Some((specified_path, path_to_config)) = try!(self.get_string(&key)) {
-            if specified_path.contains("/") || (cfg!(windows) && specified_path.contains("\\")) {
-                // An absolute or a relative path
-                let prefix_path = path_to_config.parent().unwrap().parent().unwrap();
-                // Joining an absolute path to any path results in the given absolute path
-                Ok(Some(prefix_path.join(specified_path)))
+    pub fn get_bool(&self, key: &str) -> CargoResult<Option<Value<bool>>> {
+        if let Some(v) = try!(self.get_env(key)) {
+            return Ok(Some(v))
+        }
+        match try!(self.get(key)) {
+            Some(CV::Boolean(b, path)) => {
+                Ok(Some(Value {
+                    val: b,
+                    definition: Definition::Path(path),
+                }))
+            }
+            Some(val) => self.expected("bool", key, val),
+            None => Ok(None),
+        }
+    }
+
+    pub fn get_path(&self, key: &str) -> CargoResult<Option<Value<PathBuf>>> {
+        if let Some(val) = try!(self.get_string(&key)) {
+            let is_path = val.val.contains("/") ||
+                          (cfg!(windows) && val.val.contains("\\"));
+            let path = if is_path {
+                val.definition.root(self).join(val.val)
             } else {
                 // A pathless name
-                Ok(Some(PathBuf::from(specified_path)))
-            }
+                PathBuf::from(val.val)
+            };
+            Ok(Some(Value {
+                val: path,
+                definition: val.definition,
+            }))
         } else {
             Ok(None)
         }
     }
 
-    pub fn get_list(&self, key: &str) -> CargoResult<Option<(Vec<(String, PathBuf)>, PathBuf)>> {
+    pub fn get_list(&self, key: &str)
+                    -> CargoResult<Option<Value<Vec<(String, PathBuf)>>>> {
         match try!(self.get(key)) {
-            Some(CV::List(i, path)) => Ok(Some((i, path))),
+            Some(CV::List(i, path)) => {
+                Ok(Some(Value {
+                    val: i,
+                    definition: Definition::Path(path),
+                }))
+            }
             Some(val) => self.expected("list", key, val),
             None => Ok(None),
         }
     }
 
     pub fn get_table(&self, key: &str)
-                    -> CargoResult<Option<(HashMap<String, CV>, PathBuf)>> {
+                    -> CargoResult<Option<Value<HashMap<String, CV>>>> {
         match try!(self.get(key)) {
-            Some(CV::Table(i, path)) => Ok(Some((i, path))),
+            Some(CV::Table(i, path)) => {
+                Ok(Some(Value {
+                    val: i,
+                    definition: Definition::Path(path),
+                }))
+            }
             Some(val) => self.expected("table", key, val),
             None => Ok(None),
         }
     }
 
-    pub fn get_i64(&self, key: &str) -> CargoResult<Option<(i64, PathBuf)>> {
+    pub fn get_i64(&self, key: &str) -> CargoResult<Option<Value<i64>>> {
+        if let Some(v) = try!(self.get_env(key)) {
+            return Ok(Some(v))
+        }
         match try!(self.get(key)) {
-            Some(CV::Integer(i, path)) => Ok(Some((i, path))),
+            Some(CV::Integer(i, path)) => {
+                Ok(Some(Value {
+                    val: i,
+                    definition: Definition::Path(path),
+                }))
+            }
             Some(val) => self.expected("integer", key, val),
             None => Ok(None),
         }
@@ -204,6 +269,22 @@ impl Config {
         })
     }
 
+    pub fn configure_shell(&self,
+                           verbose: Option<bool>,
+                           quiet: Option<bool>,
+                           color: &Option<String>) -> CargoResult<()> {
+        let cfg_verbose = try!(self.get_bool("term.verbose")).map(|v| v.val);
+        let cfg_color = try!(self.get_string("term.color")).map(|v| v.val);
+        let verbose = verbose.or(cfg_verbose).unwrap_or(false);
+        let quiet = quiet.unwrap_or(false);
+        let color = color.as_ref().or(cfg_color.as_ref());
+
+        try!(self.shell().set_verbosity(verbose, quiet));
+        try!(self.shell().set_color_config(color.map(|s| &s[..])));
+
+        Ok(())
+    }
+
     fn load_values(&self) -> CargoResult<()> {
         let mut cfg = CV::Table(HashMap::new(), PathBuf::from("."));
 
@@ -243,26 +324,25 @@ impl Config {
     }
 
     fn scrape_target_dir_config(&mut self) -> CargoResult<()> {
-        if let Some((dir, dir2)) = try!(self.get_string("build.target-dir")) {
-            let mut path = PathBuf::from(dir2);
-            path.pop();
-            path.pop();
-            path.push(dir);
-            *self.target_dir.borrow_mut() = Some(path);
-        } else if let Some(dir) = env::var_os("CARGO_TARGET_DIR") {
+        if let Some(dir) = env::var_os("CARGO_TARGET_DIR") {
             *self.target_dir.borrow_mut() = Some(self.cwd.join(dir));
+        } else if let Some(val) = try!(self.get_path("build.target-dir")) {
+            *self.target_dir.borrow_mut() = Some(val.val);
         }
         Ok(())
     }
 
     fn get_tool(&self, tool: &str) -> CargoResult<PathBuf> {
+        let var = tool.chars().flat_map(|c| c.to_uppercase()).collect::<String>();
+        if let Some(tool_path) = env::var_os(&var) {
+            return Ok(PathBuf::from(tool_path));
+        }
+
         let var = format!("build.{}", tool);
         if let Some(tool_path) = try!(self.get_path(&var)) {
-            return Ok(tool_path);
+            return Ok(tool_path.val);
         }
 
-        let var = tool.chars().flat_map(|c| c.to_uppercase()).collect::<String>();
-        let tool = env::var_os(&var).unwrap_or_else(|| OsString::from(tool));
         Ok(PathBuf::from(tool))
     }
 }
@@ -282,6 +362,16 @@ pub enum ConfigValue {
     Boolean(bool, PathBuf),
 }
 
+pub struct Value<T> {
+    pub val: T,
+    pub definition: Definition,
+}
+
+pub enum Definition {
+    Path(PathBuf),
+    Environment,
+}
+
 impl fmt::Debug for ConfigValue {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
@@ -464,12 +554,30 @@ impl ConfigValue {
     }
 }
 
+impl Definition {
+    pub fn root<'a>(&'a self, config: &'a Config) -> &'a Path {
+        match *self {
+            Definition::Path(ref p) => p.parent().unwrap().parent().unwrap(),
+            Definition::Environment => config.cwd(),
+        }
+    }
+}
+
+impl fmt::Display for Definition {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        match *self {
+            Definition::Path(ref p) => p.display().fmt(f),
+            Definition::Environment => "the environment".fmt(f),
+        }
+    }
+}
+
 fn homedir(cwd: &Path) -> Option<PathBuf> {
     let cargo_home = env::var_os("CARGO_HOME").map(|home| {
         cwd.join(home)
     });
     let user_home = env::home_dir().map(|p| p.join(".cargo"));
-    return cargo_home.or(user_home);
+    cargo_home.or(user_home)
 }
 
 fn walk_tree<F>(pwd: &Path, mut walk: F) -> CargoResult<()>
