@@ -21,11 +21,11 @@ impl<N: Eq + Hash + Clone> Graph<N> {
     }
 
     pub fn add(&mut self, node: N, children: &[N]) {
-        self.nodes.insert(node, children.iter().map(|n| n.clone()).collect());
+        self.nodes.insert(node, children.iter().cloned().collect());
     }
 
     pub fn link(&mut self, node: N, child: N) {
-        self.nodes.entry(node).or_insert_with(|| HashSet::new()).insert(child);
+        self.nodes.entry(node).or_insert_with(HashSet::new).insert(child);
     }
 
     pub fn get_nodes(&self) -> &HashMap<N, HashSet<N>> {
